#include "IsoMiniMap.h"

using namespace std;
using namespace graphic;

IsoMiniMap::IsoMiniMap(IsoTilesMap * tm, IsoObjectsMap * om, TileSet * ts) :
					   Map(tm->GetNumRows(), tm->GetNumCols()) 
{
	// Tile dimensions
	a_tile_width = ts->GetTileW();
	a_tile_height = ts->GetTileH();
    a_square_side = a_tile_height * SQRT2;
	int half_height = a_tile_height / 2;
	// scale of the minimap
	a_scale = tm->GetTileW() / a_tile_width;
	
	// indexes
	int r , c;
	
	// temporary Point for tiles
	Point constr_p;
	// temporary tile pointer
	MiniTile * constr_tile;

	// set the image origin so that the [num_rows, 0] tile have x = 0 and the [0,0] 
	// tile have y = 0
	a_graphic_origin.x = ((a_num_rows - 1) * a_tile_height);
	a_graphic_origin.y = 0;
	
	// origin of the map
	a_origin.x = a_graphic_origin.x + a_tile_height;
	a_origin.y = a_graphic_origin.y;
	
	// store IsoTilesMap pointer
	a_tm = tm;
	// store IsoObjectsMap pointer
	a_om = om;
	// store TileSet pointer
	a_tile_set = ts;
	
	// dimensions of the map
	a_width = (a_num_cols + a_num_rows) * a_tile_height;
	a_height = a_width / 2;
	
	// number of used images
	a_num_images = ts->GetNumImg();
	
	// number of cells and objects
	a_num_cells = a_num_cols * a_num_rows;
	a_num_objects = a_num_cells;
	
	for(r = 0; r < a_num_rows; r++)
	{
		for(c = 0; c < a_num_cols; c++)
		{
            // visible and walkable tile
            if(a_tm->IsWalkable(r,c) && a_tm->IsVisible(r,c) == VISIBLE)
                constr_tile = new MiniTile(ts->GetImage(IMG_VISIBLE));
			// partially visible tile		
            else if(a_tm->IsVisible(r,c) == PARTIALLY_VISIBLE)
                constr_tile = new MiniTile(ts->GetImage(IMG_PARTIALLY_VISIBLE));
            // visible but not walkable tile (ELEMENT)
            else if(a_tm->IsVisible(r,c) == VISIBLE)
            {
                Element * el = (Element *) om->GetObject(r, c);
                
                // scene element
                if(el->GetType() == SCENE_ELEMENT)
                    constr_tile = new MiniTile(ts->GetImage(IMG_ELEMENT));
                // faction element
                else
                {
                    int faction = (dynamic_cast <FactionElement *>(el))->GetFaction();
                    
                    if(faction == F_MERCENARIES)
                        constr_tile = new MiniTile(ts->GetImage(IMG_MERCENARIES));
                    else if(faction == F_NGG)
                        constr_tile = new MiniTile(ts->GetImage(IMG_NGG));
                    else if(faction == F_SWA)
                        constr_tile = new MiniTile(ts->GetImage(IMG_SWA));
                    else
                        constr_tile = new MiniTile(ts->GetImage(IMG_NO_FACTION));                    
                }
            }
            // not visible tile
            else
                constr_tile = new MiniTile(ts->GetImage(IMG_NOT_VISIBLE));
			
			// set indexes of the new tile
			constr_tile->SetIndexes(r,c);

			// set tile isometric position
			constr_p.x = (c - r) * a_tile_height + a_graphic_origin.x; 
			constr_p.y = (c + r) * half_height + a_graphic_origin.y; 
			constr_tile->SetPosition(constr_p);
			
			// store the tile into the tiles vector
			a_objects.push_back(constr_tile);
		}
	}
}

// blit the map on a surface
void IsoMiniMap::BlitLayer(graphic::BaseSurface & dst_surf, SDL_Rect * limits_rect)
{
	Sint16 l_limit = limits_rect->x ;
	Sint16 r_limit = limits_rect->x + limits_rect->w;
	Sint16 u_limit = limits_rect->y;
	Sint16 b_limit = limits_rect->y + limits_rect->h;	

	// temporary Point
	Point * p = new Point;
	
	Image * cur_img;
	MiniTile * cur_tile;
   
    FloatPoint * start, * end;
    int s_row, s_col, e_row, e_col;
    
    // compute start row =>  top, right corner
    start = iso2scr(r_limit, u_limit, a_origin.x, a_origin.y);            
    s_row = (int)(start->y / a_square_side) - 1;           
    if(s_row < 0)
        s_row = 0;
    delete start;
   
    // compute start col => top, left corner
    start = iso2scr(l_limit, u_limit, a_origin.x, a_origin.y);    
    s_col = (int)(start->x / a_square_side) - 1;
    if(s_col < 0)
        s_col = 0;
    delete start;    
    
    // compute end row => bottom, left corner
    end = iso2scr(l_limit, b_limit, a_origin.x, a_origin.y);    
    // row and col of the end Point
    e_row = (int)(end->y / a_square_side) + 1;
    if(e_row > a_num_rows)
        e_row = a_num_rows;
    delete end;
    
    // compute end col => bottom, right corner
    end = iso2scr(r_limit, b_limit, a_origin.x, a_origin.y);    
    e_col = (int)(end->x / a_square_side) + 1;                
    if(e_col > a_num_cols)
        e_col = a_num_cols;
    delete end;
		
    for(int r = s_row; r < e_row ; r++)
    {        
        for(int c = s_col; c < e_col; c++)
        {
            cur_tile = (MiniTile *) a_objects[(r * a_num_cols) + c];
            p->x = cur_tile->GetX();
            p->y = cur_tile->GetY();

		    if( ((p->x + a_tile_width >= l_limit) && 
		 	(p->x <= r_limit) && 
			(p->y + a_tile_height >= u_limit) && 
			(p->y <= b_limit)))
		    {	
			    cur_img = cur_tile->GetImage();
   
			    //set blit position of the image
			    cur_img->SetPosition(p->x - l_limit, p->y - u_limit);
			    //blit 
			    blitter->Blit(* cur_img, dst_surf);
		    }
        }
	}
	
	delete p;
}

// blit the map on a surface
void IsoMiniMap::BlitLayer(graphic::BaseSurface & dst_surf)
{
	
	Image * cur_img;
	MiniTile * cur_tile;
	
	for(int i = 0; i < a_num_cells; i++)
	{
		cur_tile =  (MiniTile *) a_objects[i];

			cur_img = cur_tile->GetImage();
			
			//set blit position of the image
			cur_img->SetPosition(a_objects[i]->GetX(), a_objects[i]->GetY());
			//blit 
			blitter->Blit(* cur_img, dst_surf);
	}
}

void IsoMiniMap::UpdateMap()
{
	
	for(int r = 0; r < a_num_rows; r++)
	{
		for(int c = 0; c < a_num_cols; c++)
		{   
            // visible and walkable tile   
            if(a_tm->IsWalkable(r,c) && a_tm->IsVisible(r,c) == VISIBLE)
                ((MiniTile *) a_objects[(r * a_num_cols) + c])->SetImage(a_tile_set->GetImage(IMG_VISIBLE));
			// partially visible tile
			else if(a_tm->IsVisible(r,c) == PARTIALLY_VISIBLE)
                ((MiniTile *) a_objects[(r * a_num_cols) + c])->SetImage(a_tile_set->GetImage(IMG_PARTIALLY_VISIBLE));
            // visible but not walkable tile (ELEMENT)
            else if(a_tm->IsVisible(r,c) == VISIBLE)
            {
                Element * el = (Element *) a_om->GetObject(r, c);
                
                // scene element
                if(el->GetType() == SCENE_ELEMENT)
                    ((MiniTile *) a_objects[(r * a_num_cols) + c])->SetImage(a_tile_set->GetImage(IMG_ELEMENT));
                // faction element
                else
                {                    
                    int faction = (dynamic_cast <FactionElement *>(el))->GetFaction();
                    
                    if(faction == F_MERCENARIES)
                        ((MiniTile *) a_objects[(r * a_num_cols) + c])->SetImage(a_tile_set->GetImage(IMG_MERCENARIES));
                    else if(faction == F_NGG)
                        ((MiniTile *) a_objects[(r * a_num_cols) + c])->SetImage(a_tile_set->GetImage(IMG_NGG));
                    else if(faction == F_SWA)
                        ((MiniTile *) a_objects[(r * a_num_cols) + c])->SetImage(a_tile_set->GetImage(IMG_SWA));
                    else
                        ((MiniTile *) a_objects[(r * a_num_cols) + c])->SetImage(a_tile_set->GetImage(IMG_NO_FACTION));
                }
            }
            // not visible tile
            else
                ((MiniTile *) a_objects[(r * a_num_cols) + c])->SetImage(a_tile_set->GetImage(IMG_NOT_VISIBLE));
		}
	}
	
}
