#include "IsoObjectsMap.h"
#include <iostream>

using namespace std;
using namespace graphic;

// constructor
IsoObjectsMap::IsoObjectsMap(IsoTilesMap * tm)
{
    // store the tiles map
    a_tm = tm;   
    
    // no objects at the creation
    a_num_objects = 0;
    
    for(int i = 0; i < a_tm->GetNumCells(); i++)
        a_objects_map.push_back(NULL);
    
    // number of rows of the map
    a_num_rows = a_tm->GetNumRows();
    // number of cols of the map
    a_num_cols = a_tm->GetNumCols();
}

// add an object to the Map at the [row, col] position
void IsoObjectsMap::AddObject(IsoObject * obj, int row, int col)
{
    // increment number of objects
    a_num_objects++;    
    
    // set object indexes on the tiles map
    obj->SetIndexes(row, col);
    
    // set object absolute position respect the tiles map
    obj->SetPosition(a_tm->GetTileX(row, col), a_tm->GetTileY(row, col) + 
                ((a_tm->GetTileH() / 2) * (obj->GetNumCols() + 1)) - obj->GetH());
    
    // store the object
    a_objects.push_back(obj);
    
    // update the tiles map so that tiles under the object are not walkable
    for(int r = row - (obj->GetNumRows() - 1) ; r <= row; r++)
    {
        for(int c = col + (obj->GetNumCols() - 1) ; c >= col; c--)
        {
            a_objects_map[(r * a_num_cols) + c] = obj;
            
            a_tm->SetWalkable(r, c, false);
        }
    }
    
    SortObjects();
}

// return the absolute X position of the image of the object on the map
Sint16 IsoObjectsMap::GetObjX(IsoObject * obj)
{
    return a_tm->GetTileX(obj->GetRow(), obj->GetCol());
}
        
// return the absolute Y position of the image of the object on the map
Sint16 IsoObjectsMap::GetObjY(IsoObject * obj)
{
    return a_tm->GetTileY(obj->GetRow(), obj->GetCol()) + 
           ((a_tm->GetTileH() / 2) * (obj->GetNumCols() + 1)) - obj->GetH();
}


// blit part of the objects map on a surface
void IsoObjectsMap::BlitLayer(graphic::BaseSurface & dst_surf, SDL_Rect * limits_rect) 
{
    Sint16 l_limit = limits_rect->x ;
    Sint16 r_limit = limits_rect->x + limits_rect->w;
    Sint16 u_limit = limits_rect->y;
    Sint16 b_limit = limits_rect->y + limits_rect->h;

    Image * cur_img;
    IsoObject * cur_obj;
    bool blit = false;
    
    Point * p = new Point;
    
    // remove all elements from the vector
    a_cur_objects.clear();    
    
    for (int i = 0; i < a_num_objects; i++)
    {
        cur_obj = a_objects[i];
        cur_img = cur_obj->GetImage();
        p->x = cur_obj->GetX();
        p->y = cur_obj->GetY();
        
        // check if the object is into limits
        if( (p->x + cur_img->GetW() >= l_limit) && (p->x <= r_limit) && 
            (p->y + cur_img->GetH() >= u_limit) && (p->y <= b_limit) )
        {
            // check if one tile of the object is visible
            for(int r = cur_obj->GetRow() - (cur_obj->GetNumRows() - 1) ; r <= cur_obj->GetRow(); r++)
            {
                for(int c = cur_obj->GetCol() + (cur_obj->GetNumCols() - 1) ; c >= cur_obj->GetCol(); c--)
                {
                    if(a_tm->IsVisible(r, c) == VISIBLE)
                    {
                        blit = true;
                        break;
                    }                    
                }
                
                if(blit)
                    break;
            }
            
            // blit the object
            if(blit)
            {
                // set blit position of the image
                cur_img->SetPosition(p->x - l_limit, p->y - u_limit);
                // blit 
                blitter->Blit(*cur_img, dst_surf);
                
                // reset blit flag
                blit = false;
                
                // store the object in the current vector
                a_cur_objects.push_back(cur_obj);
            }            
        }
    }
    
    delete p;
}
        
// blit the entire objects map on a surface
void IsoObjectsMap::BlitLayer(graphic::BaseSurface & dst_surf) 
{
    Image * cur_img;
    IsoObject * cur_obj;
    bool blit = false;
    
    Point * p = new Point;
    
    // remove all elements from the vector
    a_cur_objects.clear();
    

    
    for (int i = 0; i < a_num_objects; i++)
    {
        cur_obj = a_objects[i];
        cur_img = cur_obj->GetImage();
        p->x = cur_obj->GetX();
        p->y = cur_obj->GetY();

        // check if one tile of the object is visible
        for(int r = cur_obj->GetRow() - (cur_obj->GetNumRows() - 1) ; r <= cur_obj->GetRow(); r++)
        {
            for(int c = cur_obj->GetCol() + (cur_obj->GetNumCols() - 1) ; c >= cur_obj->GetCol(); c--)
            {
                if(a_tm->IsVisible(r, c) == VISIBLE)
                {
                    blit = true;
                    break;
                }
            }
             
            if(blit)
                break;
        }
            
        // blit the object
        if(blit)
        {
            // set blit position of the image
            cur_img->SetPosition(p->x, p->y);
            // blit 
            blitter->Blit(*cur_img, dst_surf);
              
            // reset blit flag
            blit = false;
               
            // store the object in the current vector
            a_cur_objects.push_back(cur_obj);
        }

    }
    
    delete p;
}

// select the object at the x,y position
IsoObject * IsoObjectsMap::SelectObject(Sint16 x, Sint16 y)
{
    // reset the current object selected
    IsoObject * obj_selected = NULL;
    
    IsoObject * cur_obj;
    Image * cur_img;

    SDL_Rect * img_rect = new SDL_Rect;
    Point * p = new Point;
    p->x = x;
    p->y = y;
    
    int num_obj = a_cur_objects.size();
    
    // 1a- test collision in all imgs rect if true go to 2
    // 1b- test pixel trans
    for(int i = 0; i < num_obj; i++)
    {
        cur_obj = a_cur_objects[i];
        cur_img = cur_obj->GetImage();
        
        img_rect->x = cur_obj->GetX();
        img_rect->y = cur_obj->GetY();
        img_rect->w = cur_img->GetW();
        img_rect->h = cur_img->GetH();
        
        // inside the image rect
        if(p->x >= img_rect->x && p->x <= (img_rect->x + img_rect->w) && 
           p->y >= img_rect->y && p->y <= (img_rect->y + img_rect->h))
        {
            // inside the image
            if(!cur_img->IsTrans(x - img_rect->x, y - img_rect->y))
                obj_selected = cur_obj;                
        }
    }
    
    // 2- test clicked cell
    if(obj_selected == NULL)
    {
        CellInd * ind = a_tm->GetCell(x,y);
        if(ind != NULL)
            obj_selected = a_objects_map[(ind->row * a_num_cols) + ind->col];
    }
    
    // result
    if(obj_selected!= NULL)
    {
        // check if one tile of the object is visible
        for(int r = obj_selected->GetRow() - (obj_selected->GetNumRows() - 1) ; 
            r <= obj_selected->GetRow(); r++)
        {
            for(int c = obj_selected->GetCol() + (obj_selected->GetNumCols() - 1) ; 
                c >= obj_selected->GetCol(); c--)
            {
                if(a_tm->IsVisible(r, c) == VISIBLE)
                    return obj_selected;
            }
        }

    }

    // no object found
    return NULL;
}

// sort objects so to blit front objects after rear objects
void IsoObjectsMap::SortObjects()
{
    IsoObject * temp;
    bool sorted = false;

    int j , i = 0;
    
    // DEBUG
    // int counter = 0;
    
    while((i < a_num_objects) && (!sorted))
    {
        sorted = true;
        
        for(j = 0; j < a_num_objects - i - 1; j++)
        {
            if(*a_objects[j] > *a_objects[j + 1]) 
            {
                temp = a_objects[j + 1];
                a_objects[j + 1] = a_objects[j];
                a_objects[j] = temp;
                
                sorted = false;
            }
            
            // DEBUG
            //counter++;
        }
        i++;
    }   
    
    // DEBUG
    //cout << "sort steps: " << counter << endl;
}


// move an object at the [row, col] position
void IsoObjectsMap::MoveObject(IsoObject * obj, int row, int col)
{
    // remove current object occupation
    for(int r = obj->GetRow() - (obj->GetNumRows() - 1) ; r <= obj->GetRow(); r++)
    {
        for(int c = obj->GetCol() + (obj->GetNumCols() - 1) ; c >= obj->GetCol(); c--)
        {
            a_objects_map[(r * a_num_cols) + c] = NULL;
            
            a_tm->SetWalkable(r, c, true);
        }            
    }

    // set object indexes on the tiles map
    obj->SetIndexes(row, col);

    // remove current object occupation
    for(int r = obj->GetRow() - (obj->GetNumRows() - 1) ; r <= obj->GetRow(); r++)
    {
        for(int c = obj->GetCol() + (obj->GetNumCols() - 1) ; c >= obj->GetCol(); c--)
        {
            a_objects_map[(r * a_num_cols) + c] = obj;
            
            a_tm->SetWalkable(r, c, false);
        }            
    }

    // re-sort objects
    SortObjects();  
}

// set the visibility range of an object in accord with the view direction and its
// visibility radius.         
void IsoObjectsMap::ComputeObjVisibility(IsoObject * obj, int radius, int direction)
{
    // row and col
    int r, c;
    // number of rows and cols occupied by the object
    int num_r, num_c;
    // visible range
    int r_start, r_end;   
    int c_start, c_end;
    // partially visible range
    int r2_start, r2_end;
    int c2_start, c2_end;    
    // partially visible radius
    int pv_radius;
    
    int row, col;
    
    r = obj->GetRow();
    c = obj->GetCol();
    num_r = obj->GetNumRows();
    num_c = obj->GetNumCols();
    
    // compute partially visible radius
    pv_radius = radius / PV_COEFFICIENT;
    if(pv_radius == 0)
        pv_radius = 1;
    
    // errors check
    if(radius < 1 || r < 0 || c < 0)
        return ;
    
    // COMPUTE VISIBILITY RANGE
    switch(direction)
    {
        case NORTH:
            r_start = r - radius - (num_r - 1);
            r_end = r;        
            c_start = c - radius;
            c_end = c + radius + (num_c - 1);              
            break;
            
        case NORTH_EAST:
            r_start = r - radius - DIAGONAL_BONUS - (num_r - 1);
            r_end = r;
            c_start = c;
            c_end = c + radius + DIAGONAL_BONUS + (num_c - 1);            
            break;
            
        case NORTH_WEST:
            r_start = r - radius - DIAGONAL_BONUS - (num_r - 1);
            r_end = r;        
            c_start = c - radius - DIAGONAL_BONUS;
            c_end = c + (num_c - 1);                 
            break;            
            
        case SOUTH:
            r_start = r;
            r_end = r + radius;
            c_start = c - radius;
            c_end = c + radius + (num_c - 1);
            break;
            
        case SOUTH_EAST:
            r_start = r;
            r_end = r + radius + DIAGONAL_BONUS;        
            c_start = c;
            c_end = c + radius + DIAGONAL_BONUS + (num_c - 1);
            break;
            
        case SOUTH_WEST:
            r_start = r;
            r_end = r + radius + DIAGONAL_BONUS;        
            c_start = c - radius - DIAGONAL_BONUS;
            c_end = c + (num_c - 1) ;
            break;            
                  
        case EAST:
            r_start = r - radius - (num_r - 1);
            r_end = r + radius;
            c_start = c;
            c_end = c + radius + (num_c - 1);
            break;
            
        case WEST:
            r_start = r - radius  - (num_r - 1);
            r_end = r + radius;       
            c_start = c - radius;
            c_end = c + (num_c - 1);   
            break;
            
        case ANY:
            r_start = r - radius - (num_r - 1);
            r_end = r + radius;
            c_start = c - radius;
            c_end = c + radius + (num_c - 1);
            break;
                        
        default:
            return ;
            break;
    }    
            
    // boundaries checks
    if(r_start < 0)
        r_start = 0;
    if(c_start < 0)
        c_start = 0;

    if(c_start >= a_num_cols)
        c_start = a_num_cols - 1;  

    if(r_end >= a_num_rows)
        r_end = a_num_rows - 1;  
    if(c_end >= a_num_cols)
        c_end = a_num_cols - 1;     
                
    // set VISIBLE range
    for(row = r_start ; row <= r_end ; row++)
        for(col = c_start ; col <= c_end ; col++)
            a_tm->SetVisibility(row, col);    
    
    // PARTIALLY VISIBLE RANGE - NORTH
    r2_start = r_start - pv_radius;
    r2_end = r_start - 1;
    c2_start = c_start ;
    c2_end = c_end;

    if(r2_start < 0)
        r2_start = 0;
    if(r2_end < 0)
        r2_end = 0;
        
    // set partially visible tiles - NORTH
    for(row = r2_start ; row <= r2_end ; row++)
        for(col = c2_start ; col <= c2_end ; col++)    
            if(a_tm->IsVisible(row, col) != VISIBLE)
                a_tm->SetVisibility(row, col, PARTIALLY_VISIBLE);              
    
    // PARTIALLY VISIBLE RANGE - EAST
    r2_start = r_start - pv_radius;
    r2_end = r_end + pv_radius;
    c2_start = c_end + 1;
    c2_end = c_end + pv_radius;
        
    if(r2_start < 0)
        r2_start = 0;
    if(r2_end >= a_num_rows)
        r2_end = a_num_rows - 1; 
    if(c2_start >= a_num_cols)
        c2_start = a_num_cols - 1;
    if(c2_end >= a_num_cols)
        c2_end = a_num_cols - 1;
        
    // set partially visible tiles - EAST
    for(row = r2_start ; row <= r2_end ; row++)
        for(col = c2_start ; col <= c2_end ; col++)    
            if(a_tm->IsVisible(row, col) != VISIBLE)
                a_tm->SetVisibility(row, col, PARTIALLY_VISIBLE);  
        
    // PARTIALLY VISIBLE RANGE - WEST    
    r2_start = r_start - pv_radius;
    r2_end = r_end + pv_radius;        
    c2_start = c_start - pv_radius;
    c2_end = c_start - 1;
        
    if(r2_start < 0)
        r2_start = 0;
    if(r2_end >= a_num_rows)
        r2_end = a_num_rows - 1; 
    if(c2_start < 0)
        c2_start = 0;
    if(c2_end < 0)
        c2_end = 0;
        
    // set partially visible tiles - WEST
    for(row = r2_start ; row <= r2_end ; row++)
        for(col = c2_start ; col <= c2_end ; col++)    
            if(a_tm->IsVisible(row, col) != VISIBLE)
                a_tm->SetVisibility(row, col, PARTIALLY_VISIBLE);  
      
    // PARTIALLY VISIBLE RANGE - SOUTH            
    r2_start = r_end + 1;
    r2_end = r_end + pv_radius;
    c2_start = c_start ;
    c2_end = c_end;
        
    if(r2_start >= a_num_rows)
        r2_start = a_num_rows - 1;  
    if(r2_end >= a_num_rows)
        r2_end = a_num_rows - 1;   
        
    // set partially visible tiles - SOUTH
    for(row = r2_start ; row <= r2_end ; row++)
        for(col = c2_start ; col <= c2_end ; col++)    
            if(a_tm->IsVisible(row, col) != VISIBLE)
                a_tm->SetVisibility(row, col, PARTIALLY_VISIBLE);    
}

