#include "IsoTilesMap.h"

using namespace std;
using namespace graphic;

// base constructor, use a map to choose tiles
IsoTilesMap::IsoTilesMap(int num_rows, int num_cols, int *map, TileSet * ts):
				   Map(num_rows, num_cols) 
{
	// Tile dimensions
	a_tile_width = ts->GetTileW();
	a_tile_height = ts->GetTileH();
	a_square_side = a_tile_height * SQRT2;
	int half_height = a_tile_height / 2;
	
	// indexes
	int r , c;
	
	// temporary Point for tiles
	Point constr_p;
	// temporary tile pointer
	Tile * constr_tile;
	
	// set the image origin so that the [num_rows, 0] tile have x = 0 and the [0,0] 
	// tile have y = 0
	a_graphic_origin.x = ((num_rows - 1) * a_tile_height);
	a_graphic_origin.y = 0;
	
	// origin of the map
	a_origin.x = a_graphic_origin.x + a_tile_height;
	a_origin.y = a_graphic_origin.y;
	
	// storing TileSet pointer
	a_tile_set = ts;
	
	// dimensions of the map
	a_width = (num_cols + num_rows) * a_tile_height;
	a_height = a_width / 2;
	
	// number of used images
	a_num_images = ts->GetNumImg();
	// number of cells and objects
	a_num_cells = num_cols * num_rows;
	a_num_objects = a_num_cells;
	
	for(r = 0; r < a_num_rows; r++)
	{
		for(c = 0; c < a_num_cols; c++)
		{
			// create a new tile
			constr_tile = new Tile(ts->GetImage(*(map + ((r * num_cols) + c))));
			// set indexes of the new tile
			constr_tile->SetIndexes(r,c);

			// set tile isometric position
			constr_p.x = (c - r) * a_tile_height + a_graphic_origin.x; 
			constr_p.y = (c + r) * half_height + a_graphic_origin.y; 
			constr_tile->SetPosition(constr_p);
			
			// store the tile into the tiles vector
			a_objects.push_back(constr_tile);
		}
	}
}

// random map generation constructor, no map required
IsoTilesMap::IsoTilesMap(int num_rows, int num_cols, TileSet * ts):
				   Map(num_rows, num_cols) 
{
	// Tile dimensions
	a_tile_width = ts->GetTileW();
	a_tile_height = ts->GetTileH();
	a_square_side = a_tile_height * SQRT2;
	int half_height = a_tile_height / 2;
	
	// indexes rows/cols
	int r , c;
	// random index 
	int ind_img;
	
	// temporary Point for tiles
	Point constr_p;
	// temporary tile pointer
	Tile * constr_tile;

	// init random generator
	srand(time(NULL));
	
	// set the origin so that the [num_rows, 0] tile have x = 0 and the [0,0] 
	// tile have y = 0
	a_graphic_origin.x = ((num_rows - 1) * a_tile_height);
	a_graphic_origin.y = 0;
	
	// origin of the map
	a_origin.x = a_graphic_origin.x + a_tile_height;
	a_origin.y = a_graphic_origin.y;
	
	// storing TileSet pointer
	a_tile_set = ts;
	
	// dimensions of the map
	a_width = (num_cols + num_rows) * a_tile_height;
	a_height = a_width / 2;

	// number of used images
	a_num_images = ts->GetNumImg();
	// number of cells and objects
	a_num_cells = num_cols * num_rows;
	a_num_objects = a_num_cells;

	for(r = 0; r < a_num_rows; r++)
	{
		for(c = 0; c < a_num_cols; c++)
		{
			ind_img = (rand() % a_num_images);

			// create a new tile
			constr_tile = new Tile(ts->GetImage(ind_img));
			// set indexes of the new tile
			constr_tile->SetIndexes(r,c);

			// set tile isometric position
			constr_p.x = (c - r) * a_tile_height + a_graphic_origin.x; 
			constr_p.y = (c + r) * half_height + a_graphic_origin.y; 
			constr_tile->SetPosition(constr_p);
			
			// adding it to tiles vector
			a_objects.push_back(constr_tile);
		}
	}
}

// blit the map on a surface
void IsoTilesMap::BlitLayer(graphic::BaseSurface & dst_surf, SDL_Rect * limits_rect)
{
	Sint16 l_limit = limits_rect->x ;
	Sint16 r_limit = limits_rect->x + limits_rect->w;
	Sint16 u_limit = limits_rect->y;
	Sint16 b_limit = limits_rect->y + limits_rect->h;	
	
	Image * cur_img;
	Tile * cur_tile;
		
    Point * p = new Point;
    
    FloatPoint * start, * end;
    int s_row, s_col, e_row, e_col;
    
    // compute start row =>  top, right corner
    start = iso2scr(r_limit, u_limit, a_origin.x, a_origin.y);            
    s_row = (int)(start->y / a_square_side) - 1;           
    if(s_row < 0)
        s_row = 0;
    delete start;
   
    // compute start col => top, left corner
    start = iso2scr(l_limit, u_limit, a_origin.x, a_origin.y);    
    s_col = (int)(start->x / a_square_side) - 1;
    if(s_col < 0)
        s_col = 0;
    delete start;    
    
    // compute end row => bottom, left corner
    end = iso2scr(l_limit, b_limit, a_origin.x, a_origin.y);    
    // row and col of the end Point
    e_row = (int)(end->y / a_square_side) + 1;
    if(e_row > a_num_rows)
        e_row = a_num_rows;
    delete end;
    
    // compute end col => bottom, right corner
    end = iso2scr(r_limit, b_limit, a_origin.x, a_origin.y);    
    e_col = (int)(end->x / a_square_side) + 1;                
    if(e_col > a_num_cols)
        e_col = a_num_cols;
    delete end;

    for(int r = s_row; r < e_row ; r++)
    {        
        for(int c = s_col; c < e_col; c++)
        {
            cur_tile =  (Tile *) a_objects[(r * a_num_cols) + c];
            p->x = cur_tile->GetX();
            p->y = cur_tile->GetY();
        
            //check if this tile need to be blitted
            if( cur_tile->IsVisible() &&
               (p->x + a_tile_width >= l_limit) && (p->x <= r_limit) && 
               (p->y + a_tile_height >= u_limit) && (p->y <= b_limit))                            
            {   
                cur_img = cur_tile->GetImage();

                //set blit position of the image
                cur_img->SetPosition(p->x - l_limit, p->y - u_limit );
                //blit 
                blitter->Blit(* cur_img, dst_surf);
            }  
        }
    }
 
    delete p;
}

// blit the map on a surface
void IsoTilesMap::BlitLayer(graphic::BaseSurface & dst_surf)
{
	
	Image * cur_img;
	Tile * cur_tile;
		
	for(int i = 0; i < a_num_cells; i++)
	{
		cur_tile =  (Tile *) a_objects[i];

		if( cur_tile->IsVisible())
		{	
			cur_img = cur_tile->GetImage();
			
			//set blit position of the image
			cur_img->SetPosition(a_objects[i]->GetX(), a_objects[i]->GetY());
			//blit 
			blitter->Blit(* cur_img, dst_surf);
		}
	}
}

// check if a Point is inside the map passing 2 int
bool IsoTilesMap::IsInsideMap(Sint16 x, Sint16 y)
{
	FloatPoint * scr;
	int row, col;
	Point iso;
	
	iso.x = x;
	iso.y = y;
	
	scr = iso2scr(iso, a_origin);
	
	// outside the map
	if(scr->x < 0 || scr->y < 0)
	{
		delete scr;
		return false;
	}
	
	// row and col of the Point
	row = (int)(scr->y / a_square_side);
	col = (int)(scr->x / a_square_side);

	delete scr;
	
	// outside the map
	if(row >= a_num_rows || col >= a_num_cols)
		return false;
	
	// inside the map
	return true;
}

// return the cell indexes of the selected point if the selection is inside the 
// map, NULL otherwise
CellInd * IsoTilesMap::GetCell(Sint16 x, Sint16 y)
{
    FloatPoint * scr;
    Point iso;
    CellInd * ind = new CellInd;
    
    iso.x = x;
    iso.y = y;
    
    scr = iso2scr(iso, a_origin);
    
    // outside the map
    if(scr->x < 0 || scr->y < 0)
    {
        delete scr;
        delete ind;
        
        return NULL;
    }
    
    // row and col of the Point
    ind->row = (int)(scr->y / a_square_side);
    ind->col = (int)(scr->x / a_square_side);

    delete scr;
    
    // outside the map
    if(ind->row >= a_num_rows || ind->col >= a_num_cols)
        return NULL;
    
    // inside the map, return indexes
    return ind;
}

