#include "SceneRenderer.h"
#include <iostream>

using namespace std;
using namespace graphic;

// constructor
SceneRenderer::SceneRenderer(SDL_Rect & view)
{
    // set initial view
    a_view = new SDL_Rect;
    a_view->x = view.x;
    a_view->y = view.y;
    a_view->w = view.w;
    a_view->h = view.h; 
    
    // set default limits
    a_u_limit = 0;        
    a_b_limit = view.h;
    a_l_limit = 0;
    a_r_limit = view.w;
    
    // create buffer
    a_buffer = new Surface(view.w, view.h);
    
    // default fill color is black
    a_fill_color.r = 0;
    a_fill_color.g = 0;
    a_fill_color.b = 0;
    
    // no scrolling at the beginning
    _scrolling = false;
    
    _scroll_speed = SCROLL_SPEED;
    
    //_scroll_mov = 0;
    _scroll_med_mov = 0;
    _scroll_tot_mov = 0;
    _mov_times = 0;
        
    _scrolling_dir.reset();
}

// blit the current scene on the dest surface
void SceneRenderer::BlitScene(BaseSurface & dest)
{
    int num_layers = a_layers.size();
    
    // clear the dest
    dest.Fill(a_fill_color);    
    
    // blit layers
    for(int i = 0; i < num_layers; i++)    
    {
        if(a_layers[i]->NeedBlit()) 
            a_layers[i]->BlitLayer(dest, a_view);
    }
       
    a_need_blit = false;
}

// blit the current scene, using the buffer, on the dest surface
void SceneRenderer::BufferedBlitScene(BaseSurface & dest)
{
    int num_layers = a_layers.size();
    
    // clear the dest
    dest.Fill(a_fill_color);
    
    // blit the buffer
    blitter->Blit(*a_buffer, dest);
    
    // blit other layers
    for(int i = 0; i < num_layers; i++)    
    {
        if(a_layers[i]->NeedBlit() && !a_layers[i]->IsBuffered())
            a_layers[i]->BlitLayer(dest,  a_view);

    }
    
    a_need_buf_blit = false;
}
     
// refresh the buffer
void SceneRenderer::RefreshBuffer()
{
    int num_layers = a_layers.size();
    
    // clear the buffer
    a_buffer->Fill(a_fill_color);
    
    // buffer layers
    for(int i = 0; i < num_layers; i++)    
        if(a_layers[i]->NeedBlit() && a_layers[i]->IsBuffered())   
            a_layers[i]->BlitLayer(*a_buffer,  a_view);
    
    a_need_buf_refresh = false;
}
          
// add a layer to the blit vector
void SceneRenderer::AddLayer(Layer * layer)
{
    // set layer id
    layer->SetLevel(a_layers.size());
    
    a_layers.push_back(layer);    
}

// move a layer up in the blit vector
void SceneRenderer::MoveLayerUp(int id)
{
    int num_layers = a_layers.size();
    int i;
    Layer * layer;    
    
    for(i = 0; i < num_layers; i++)
    {
        if(a_layers[i]->GetLevel() == id)                    
            break;        
    }
    
    // last layer => do nothing
    if(i == (num_layers - 1))
        return ;
                
    // swap layers
    layer = a_layers[i];
    a_layers[i] = a_layers[i + 1];
    a_layers[i + 1] = layer;
    // update level id
    a_layers[i]->DownLevel();
    a_layers[i + 1]->UpLevel();
}

// move a layer down in the blit vector
void SceneRenderer::MoveLayerDown(int id)
{
    int num_layers = a_layers.size();
    int i;
    Layer * layer;    
    
    for(i = 0; i < num_layers; i++)
    {
        if(a_layers[i]->GetLevel() == id)                    
            break;        
    }
    
    // first layer => do nothing
    if(!i)
        return ;
    
    // swap layers
    layer = a_layers[i];
    a_layers[i] = a_layers[i - 1];
    a_layers[i - 1] = layer;
    // update level id
    a_layers[i]->UpLevel();
    a_layers[i - 1]->DownLevel();
}

// update view position
void SceneRenderer::UpdateViewPosition(Sint16 x, Sint16 y)
{
    a_view->x = x;
    a_view->y = y;                                                            
    
    // check for horizontal limits exceeded
    if(a_view->x < a_l_limit)
        a_view->x = a_l_limit; 
    if(a_view->x > a_r_limit)
        a_view->x = a_r_limit;
    
    // check for vertical limits exceeded
    if(a_view->y < a_u_limit)
        a_view->y = a_u_limit;
    if(a_view->y > a_b_limit)
        a_view->y = a_b_limit;
}

// move the a_view according with passed direction flags 
void SceneRenderer::MoveView(bitset<4> direction, int mov)
{
    // scrolling up
    if(direction.test(UP))
    {                
        a_view->y -= mov;
        
        if(a_view->y < a_u_limit)        
            a_view->y = a_u_limit;
    }

    // scrolling down
    if(direction.test(DOWN))
    {                
        a_view->y += mov;
        
        if(a_view->y > a_b_limit)        
            a_view->y = a_b_limit;
    }
    
    // scrolling left
    if(direction.test(LEFT))
    {                
        a_view->x -= mov;
        
        if(a_view->x < a_l_limit)        
            a_view->x = a_l_limit;
    }
    
    // scrolling right
    if(direction.test(RIGHT))
    {                
        a_view->x += mov;
        
        if(a_view->x > a_r_limit)        
            a_view->x = a_r_limit;
    }
}

// update scrolling direction according to mouse position
void SceneRenderer::MouseScrolling(Sint16 x, Sint16 y)
{
    if(x < SCROLLING_H_LIMIT)    
        _scrolling_dir.set(LEFT);        
    else if(_scrolling)            
        _scrolling_dir.set(LEFT, 0);
                
    if(x > a_view->w - SCROLLING_H_LIMIT)    
        _scrolling_dir.set(RIGHT);        
    else if(_scrolling)            
        _scrolling_dir.set(RIGHT, 0);

    if(y < SCROLLING_V_LIMIT)    
        _scrolling_dir.set(UP);        
    else if(_scrolling)                
        _scrolling_dir.set(UP, 0);
                
    if(y > a_view->h - SCROLLING_V_LIMIT)    
        _scrolling_dir.set(DOWN);     
    else if(_scrolling)                                
        _scrolling_dir.set(DOWN, 0);
    
    // scrolling 
    if(_scrolling_dir.any())
        _scrolling = true;
    // end scrolling
    else if(_scrolling) 
    {
        a_need_buf_refresh = true;
        a_need_buf_blit = true;
                    
        _scrolling = false;
                    
        _mov_times = 0;
        _scroll_tot_mov = 0;  
    }
}

// manage the scrolling each frame
void SceneRenderer::Scrolling(int time_diff)
{
    // count movement times
    _mov_times++;

    int _scroll_mov = (int)ceil(_scroll_speed * time_diff);
   
    // count total movement       
    _scroll_tot_mov += _scroll_mov;            
                
    // compute movement media
    _scroll_med_mov = _scroll_tot_mov / _mov_times;      
            
    a_need_blit = true;
    
    // scrolling up
    if(_scrolling_dir.test(UP))
    {                
        a_view->y -= _scroll_mov;
        
        if(a_view->y < a_u_limit)        
            a_view->y = a_u_limit;
    }

    // scrolling down
    if(_scrolling_dir.test(DOWN))
    {                
        a_view->y += _scroll_mov;
        
        if(a_view->y > a_b_limit)        
            a_view->y = a_b_limit;
    }
    
    // scrolling left
    if(_scrolling_dir.test(LEFT))
    {                
        a_view->x -= _scroll_mov;
        
        if(a_view->x < a_l_limit)        
            a_view->x = a_l_limit;
    }
    
    // scrolling right
    if(_scrolling_dir.test(RIGHT))
    {                
        a_view->x += _scroll_mov;
        
        if(a_view->x > a_r_limit)        
            a_view->x = a_r_limit;
    }
}
