#ifndef CLASS_SCENE_RENDERER
#define CLASS_SCENE_RENDERER

#include <cmath>
#include <bitset>
#include <vector>
#include <SDL.h>

#include "graphic/BaseSurface.h"
#include "graphic/Blitter.h"
#include "graphic/Surface.h"
#include "Layer.h"

// types of rendering
#define NORMAL 0
#define BUFFERED 1

// directions
#define UP 0
#define DOWN 1
#define LEFT 2
#define RIGHT 3

#define SCROLLING_H_LIMIT 5
#define SCROLLING_V_LIMIT 5

// scrolling data
#define SCROLL_SPEED 0.40f

/// this class manage the rendering of the current game scene
class SceneRenderer
{
	private:
     /// current view rect
     SDL_Rect * a_view;
     
     /// upper view limit
     Sint16 a_u_limit;
     /// bottom view limit
     Sint16 a_b_limit;
     /// left view limit
     Sint16 a_l_limit;
     /// right view limit
     Sint16 a_r_limit;	

     /// vector of layers to blit
     std::vector< Layer * > a_layers;
     
     /// buffer surface;
     graphic::Surface * a_buffer;
     
     /// flag that is true if the scene need to be blitted
     bool a_need_blit;
     /// flag that is true if the scene need to be blitted using the buffer
     bool a_need_buf_blit;
     /// flag that is true if the buffer need to be refreshed
     bool a_need_buf_refresh;
     /// background color of the scene
     SDL_Color a_fill_color;

     /// this flag is true during the scene scrolling
     bool _scrolling;
     /// scrolling directions
     std::bitset<4> _scrolling_dir;
     /// scrolling speed
     float _scroll_speed;        
     /// pixels of a scrolling movement
     //int _scroll_mov;
     /// medium pixels movement of the scrolling
     int _scroll_med_mov;
     /// total number of pixels movement during the scrolling
     Uint32 _scroll_tot_mov;
     /// number of scrolling movements
     Uint32 _mov_times;
     
	public:
     /// constructor
     SceneRenderer(SDL_Rect & view);
     /// destroyer
     ~SceneRenderer() 
     { 
         delete a_buffer;
         delete a_view;
     };
     
     /// blit the current scene on the dest surface
     void BlitScene(graphic::BaseSurface & dest);
     /// blit the current scene, using the buffer, on the dest surface
     void BufferedBlitScene(graphic::BaseSurface & dest);
     
     /// refresh the buffer
     void RefreshBuffer();
          
     /// add a layer to the blit vector
     void AddLayer(Layer * layer);     
     /// move a layer up in the blit vector
     void MoveLayerUp(int id);
     /// move a layer down in the blit vector
     void MoveLayerDown(int id);
     
     /// set view rectangle
     void SetView(SDL_Rect & view)
     {
         a_view->x = view.x;
         a_view->y = view.y;
         a_view->w = view.w;
         a_view->h = view.h;  
     };     
     /// update view position
     void UpdateViewPosition(Sint16 x, Sint16 y);
     /// update view dimensions
     void UpdateViewDimensions(Uint16 w, Uint16 h)
     {
         a_view->w = w;
         a_view->h = h;  
     };
     
     /// set view limits
     void SetLimits(Sint16 u_limit, Sint16 b_limit, Sint16 l_limit, Sint16 r_limit)
     {
         a_u_limit = u_limit;        
         a_b_limit = b_limit;
         a_l_limit = l_limit;
         a_r_limit = r_limit;  
     };

     /// move the view according with passed direction flags 
     void MoveView(std::bitset<4> direction, int mov);
     
     /// request of blit, is possible set the type of blit NORMAL or BUFFERED 
     /// normal is the default
     void RequestBlit(int type = NORMAL)
     {
         if(type == NORMAL)
             a_need_blit = true;
         else
             a_need_buf_blit = true;
     };
     /// return true if the sceen need to be blitted (NORMAL blit)
     bool NeedBlit() { return a_need_blit; };    
     /// return true if the sceen need to be blitted (BUFFERED blit)
     bool NeedBufferedBlit() { return a_need_buf_blit; };
     
     /// request of refresh the buffer
     void RequestBufferRefresh() { a_need_buf_refresh = true; };
     /// return true if the buf need to be refreshed
     bool NeedBufferRefresh() { return a_need_buf_refresh; };
     
     /// return the X value of the view rect
     Sint16 GetViewX() { return a_view->x; };
     /// return the Y value of the view rect
     Sint16 GetViewY() { return a_view->y; };

     /// update scrolling direction according to mouse position
     void MouseScrolling(Sint16 x, Sint16 y);     
     /// return true if the scene is scrolling
     bool IsScrolling() { return _scrolling; };     
     /// manage the scrolling each frame
     void Scrolling(int time_diff);
     /// set scrolling speed
     void SetScrollingSpeed(float speed) { _scroll_speed = speed; };

};

#endif
